package com.nisum.webflux_poc.controller;

import com.nisum.webflux_poc.dto.EmployeeDto;
import com.nisum.webflux_poc.service.KafkaProducerService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;
import com.nisum.webflux_poc.service.EmployeeService;

@RestController
@RequestMapping("/employee")
public class EmployeeController {

    private static final Logger log= LoggerFactory.getLogger(EmployeeController.class);
    @Autowired
    EmployeeService employeeService;

    @Autowired
    KafkaProducerService kafkaProducerService;

    @PostMapping("/save")
    public Mono<ResponseEntity<EmployeeDto>> saveEmployee(@RequestBody EmployeeDto employeeDto){
        log.info("Received request to save employee: {}", employeeDto);
        return employeeService.saveEmployeeData(Mono.just(employeeDto))
                .map(savedEmployee -> ResponseEntity.ok(savedEmployee))
                .defaultIfEmpty(ResponseEntity.badRequest().build())
                .onErrorResume(e -> {
                    log.error("Failed to save employee: {}", e.getMessage());
                    return Mono.just(ResponseEntity.status(500).build());
                });

    }

    @GetMapping
    public ResponseEntity<Flux<EmployeeDto>> getEmployee(){

        log.info("getEmployee details");
        return ResponseEntity.ok(employeeService.getAllEmployees());
    }

    @PutMapping("/update/{id}")
    public ResponseEntity<Mono<EmployeeDto>> updateEmployee(@RequestBody EmployeeDto employeeDto, @PathVariable String id){
        log.info("updateProduct controller is called: {}, id");
        return ResponseEntity.ok(employeeService.updateEmployee(Mono.just(employeeDto),id));
    }

    @DeleteMapping("/delete/{id}")
    public ResponseEntity<Mono<Void>> deleteEmployee(@PathVariable String id){

        return ResponseEntity.ok(employeeService.deleteEmployee(id));
    }

    @GetMapping("/{id}")
    public ResponseEntity<Mono<EmployeeDto>> getEmployee(@PathVariable String id){
        return ResponseEntity.ok(employeeService.getEmployee(id));
    }

    @GetMapping("/average/salary")
    public ResponseEntity<Flux<Double>> getAverageSalary(){

        return ResponseEntity.ok(employeeService.getAverageSalary());

    }

    @GetMapping("/kafka/send/{message}")
    public String sendDataToKafkaTopic(@PathVariable String message){
        return kafkaProducerService.sendMessage(message);

    }

    }

