package com.nisum.webflux_poc.controller;

import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

import java.time.Duration;
import java.util.List;
import java.util.Random;
import java.util.function.Function;

public class FluxAndMOnoGeneratorService {

    public Flux<String> namesFlux(){
        return Flux.fromIterable(List.of("Manasa","Nisum"))
                .log();
    }

    public Mono<String> nameMono(){

        return Mono.just("Manasa")
                .log();
    }

    public Flux<String> namesFluxMap(){
        return Flux.just("name","manasa")
                .map(String::toUpperCase)
                .log();
    }

    public Flux<String> namesFluxMapImmutable(){
        var namesFlux= Flux.fromIterable(List.of("name","manasa"));
                 namesFlux.map(String::toUpperCase)
                        .log();
                 return  namesFlux;
    }

    public Flux<String> namesFluxFlatMap(int length){
        return Flux.fromIterable(List.of("Manasa","Nisum"))
                .map(String::toUpperCase)
                .filter(s -> s.length()>length)
                .flatMap(s -> splitString(s))
                .log();
    }

    public Flux<String> namesFluxFlatMapAsync(int length){
        return Flux.fromIterable(List.of("Manasa","Nisum"))
                .map(String::toUpperCase)
                .filter(s -> s.length()>length)
                .flatMap(s -> splitStringWithDelay(s))
                .log();
    }

    public Flux<String> namesFluxConcatMap(int length){
        return Flux.fromIterable(List.of("Manasa","Nisum"))
                .map(String::toUpperCase)
                .filter(s -> s.length()>length)
                .concatMap(s -> splitStringWithDelay(s))
                .log();
    }

    public Mono<List<String>> namesMonoFlatMap(int length){
        return Mono.just("Manasa")
                .map(String::toUpperCase)
                .filter(s -> s.length()>length)
                .flatMap(this::splitStringMono)
                .log();
    }

    public Flux<String> namesFluxTransform(int length){
        Function<Flux<String>, Flux<String>> filtermap = name -> name.map(String::toUpperCase)
                .filter(s -> s.length()>length);
        return Flux.fromIterable(List.of("Manasa","Nisum"))
                .transform(filtermap)
                .flatMap(s -> splitString(s))
                .log();
    }

    public Flux<String> namesFluxTransformSwitchIfEmpty(int length){
        Function<Flux<String>, Flux<String>> filtermap = name ->
                name.map(String::toUpperCase)
                .filter(s -> s.length()>length)
                .flatMap(s->splitString(s));
        var defaultflux =  Flux.just("default")
                .transform(filtermap);

        return Flux.fromIterable(List.of(""))
                .transform(filtermap).
                switchIfEmpty(defaultflux)
                .log();
    }

    public Flux<String> exploreConcat(){
       var data= Flux.just("A","B","C");

        var data1= Flux.just("D","E","F");
        return Flux.concat(data,data1).log();
    }

    public Flux<String> exploreConcatWithMono(){
        var data= Mono.just("A");

        var data1= Mono.just("D");
        return data.concatWith(data1).log();
    }

    public Flux<String> exploreMerge(){
        var data= Flux.just("A","B","C")
                .delayElements(Duration.ofMillis(100));

        var data1= Flux.just("D","E","F")
                .delayElements(Duration.ofMillis(125));
        return Flux.merge(data,data1).log();
    }

    public Flux<String> exploreMergeWithMono(){
        var data= Mono.just("A");

        var data1= Flux.just("D");
        return data.mergeWith(data1).log();
    }


    public Flux<String> exploreMergeSequential(){
        var data= Flux.just("A","B","C")
                .delayElements(Duration.ofMillis(100));

        var data1= Flux.just("D","E","F")
                .delayElements(Duration.ofMillis(125));
        return Flux.mergeSequential(data,data1).log();
    }

    public Flux<String> exploreZip(){
        var data= Flux.just("A","B","C");

        var data1= Flux.just("D","E","F");
        return Flux.zip(data,data1, (a,b) -> a+b)
                .log(); //AD BE CF
    }

    public Flux<String> exploreZip1(){
        var data= Flux.just("A","B","C");

        var data1= Flux.just("D","E","F");
        var data2= Flux.just("1","2","3");

        var data3= Flux.just("4","5","6");

        return Flux.zip(data,data1,data2,data3)
                .map(t4 ->t4.getT1()+t4.getT2()+t4.getT3()+t4.getT4()).log(); //AD BE CF
    }

    public Flux<String> exploreZipWith(){
        var data= Flux.just("A","B","C");

        var data1= Flux.just("D","E","F");

        return data.zipWith(data1 ,(a,b) -> a+b).log(); //AD BE CF
    }
    public Flux<String> namesMonoFlatMapMany(int length){
        return Mono.just("Manasa")
                .map(String::toUpperCase)
                .filter(s -> s.length()>length)
                .flatMapMany(this::splitString)
                .log();
    }

    public Mono<String> namesMonoMap(int length){
        return Mono.just("name")
                .map(String::toUpperCase)
                .filter(s -> s.length()>4)
                .log();
    }

    public Mono<List<String>> splitStringMono(String name){
        var charArray = name.split("");
        var charList = List.of(charArray);
        return  Mono.just(charList);
    }
    public Flux<String> splitString(String name){
        var charArray = name.split("");
        return  Flux.fromArray(charArray);
    }

    public Flux<String> splitStringWithDelay(String name){
        var charArray = name.split("");
//        var delay=new Random().nextInt();
        var delay=1000;
        return  Flux.fromArray(charArray)
                .delayElements(Duration.ofMillis(delay));
    }
    public static void main(String[] args){
        FluxAndMOnoGeneratorService fluxAndMOnoGeneratorService=new FluxAndMOnoGeneratorService();
        fluxAndMOnoGeneratorService.namesFlux()
                .subscribe(name -> System.out.println(name));
        fluxAndMOnoGeneratorService.nameMono()
                .subscribe(name -> System.out.println(name));
        fluxAndMOnoGeneratorService.namesFluxMap()
                .subscribe(name -> System.out.println(name));
        fluxAndMOnoGeneratorService.namesFluxMapImmutable()
                .subscribe(name -> System.out.println(name));
    }
}
