package com.nisum.webflux_poc.service;

import com.nisum.webflux_poc.model.Employee;
import com.nisum.webflux_poc.model.SalaryAggregation;
import com.nisum.webflux_poc.utills.AppUtils;
import com.nisum.webflux_poc.dto.EmployeeDto;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.mongodb.core.ReactiveAggregationOperation;
import org.springframework.data.mongodb.core.ReactiveMongoTemplate;
import org.springframework.data.mongodb.core.aggregation.Aggregation;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;
import com.nisum.webflux_poc.repository.EmployeeRepository;
import org.springframework.stereotype.Service;

import java.util.Map;

import static org.springframework.data.mongodb.core.aggregation.Aggregation.group;


@Service
public class EmployeeService{


    @Autowired
   private EmployeeRepository employeeRepository;

    @Autowired
    ReactiveMongoTemplate mongoTemplate;

    private static final Logger log = LogManager.getLogger(EmployeeService.class);

   public Mono<EmployeeDto> saveEmployeeData(Mono<EmployeeDto> employeeMono){

       return employeeMono.map(AppUtils::dtoToentity)
               .flatMap(employeeRepository::insert)
               .map(AppUtils::entityToDto)
               .doOnNext(savedData -> log.info("Data saved successfully: {}", savedData))
               .doOnError(error -> log.error("Error occured while saving employee information: {}", error.getMessage()));
   }

    public Flux<EmployeeDto> getAllEmployees() {

        return employeeRepository.findAll()
                .map(AppUtils::entityToDto)
                .doOnNext(data -> log.info("data from database: {}", data))
                .doOnError(error -> log.error("Failed to fetch data from database: {}", error.getMessage()));
    }

    public Mono<EmployeeDto> updateEmployee(Mono<EmployeeDto> employeeDtoMono, String id){
        log.info("service method called ...");
        return employeeRepository.findById(id)
                .flatMap(p->employeeDtoMono.map(AppUtils::dtoToentity)
                        .doOnNext(e->e.setName(e.getName())))
                .flatMap(employeeRepository::save)
                .map(AppUtils::entityToDto)
                .doOnNext(data -> log.info("updated data in database for id: {} with data {}", id , data))
                .doOnError(error -> log.error("Failed to update data in database: {}", error.getMessage()));

    }

    public Mono<Void> deleteEmployee(String id){
       log.info("delete data with id: {}", id);
        return employeeRepository.deleteById(id);
    }

    public Mono<EmployeeDto> getEmployee(String id){
        return employeeRepository.findById(id)
                .map(AppUtils::entityToDto)
                .doOnNext(data -> log.info("Fetch data from database for id: {} with data {}", id , data))
                .doOnError(error -> log.error("Failed to fetch data from database wit id: {}",id, error.getMessage()));
    }

    public Flux<Double> getAverageSalary(){

        Aggregation aggregation = Aggregation.newAggregation(
                group().avg("salary").as("averageSalary")
        );

        return mongoTemplate.aggregate(aggregation, "employee", Map.class)
                .map(result -> (Double) result.get("averageSalary"))
                .defaultIfEmpty(0.0);

    }

    public Mono<EmployeeDto> sendDataToKafkaTopic(String id){
       Mono<EmployeeDto> employeeDtoMono=getEmployee(id);
       return null;
    }

}
