package com.nisum.webflux_poc;

import com.nisum.webflux_poc.controller.FluxAndMOnoGeneratorService;
import org.junit.jupiter.api.Test;
import reactor.test.StepVerifier;

import java.util.List;

public class FluxAndMonoGeneratorServiceTest {

    FluxAndMOnoGeneratorService fluxAndMOnoGeneratorService =
            new FluxAndMOnoGeneratorService();

    @Test
    void namesFlux() {

       var namesFlux= fluxAndMOnoGeneratorService.namesFlux();
        StepVerifier.create(namesFlux)
                .expectNext("Manasa","Nisum")
                .verifyComplete();
    }

    @Test
    void namesMono(){
        var namesMono = fluxAndMOnoGeneratorService.nameMono();
        StepVerifier.create(namesMono)
                .expectNext("Manasa")
                .verifyComplete();
    }

    @Test
    void namesFluxMap(){
        var upperCase = fluxAndMOnoGeneratorService.namesFluxMap();
        StepVerifier.create(upperCase)
                .expectNext("NAME","MANASA")
                .verifyComplete();
    }

    @Test
    void namesFluxMapImmutable(){
        var upperCase = fluxAndMOnoGeneratorService.namesFluxMapImmutable();
        StepVerifier.create(upperCase)
                .expectNext("name","manasa")
                .verifyComplete();
    }

    @Test
    void namesFluxFlatMap() {

        int length = 3;

        var namesFlux = fluxAndMOnoGeneratorService.namesFluxFlatMap(length);
        StepVerifier.create(namesFlux)
                .expectNext("M","A","N","A","S","A","N","I","S","U","M")
                .verifyComplete();
    }

    @Test
    void namesFluxFlatMapAsync() {

        int length = 3;

        var namesFlux = fluxAndMOnoGeneratorService.namesFluxFlatMapAsync(length);
        StepVerifier.create(namesFlux)
                .expectNextCount(11)
                .verifyComplete();
    }

    @Test
    void namesConcatMap() {

        int length = 3;

        var namesFlux = fluxAndMOnoGeneratorService.namesFluxConcatMap(length);
        StepVerifier.create(namesFlux)
                .expectNext("M","A","N","A","S","A","N","I","S","U","M")
                .verifyComplete();
    }

    void namesMonoFlatMap(){
        int length = 4;
        var value = fluxAndMOnoGeneratorService.namesMonoFlatMap(length);
        StepVerifier.create(value)
                .expectNext(List.of("M","A","N","A","S","A"))
                .verifyComplete();
    }

    @Test
    void namesMonoFlatMapMany(){
        int length = 4;
        var value = fluxAndMOnoGeneratorService.namesMonoFlatMapMany(length);
        StepVerifier.create(value)
                .expectNext("M","A","N","A","S","A")
                .verifyComplete();
    }

    @Test
    void namesFluxTransform(){
        int length = 3;

        var namesFlux = fluxAndMOnoGeneratorService.namesFluxTransform(length);
        StepVerifier.create(namesFlux)
                .expectNext("M","A","N","A","S","A","N","I","S","U","M")
                .verifyComplete();
    }

    @Test
    void namesFluxTransformSwitchIfEmpty(){

        int length = 3;

        var namesFlux = fluxAndMOnoGeneratorService.namesFluxTransformSwitchIfEmpty(length);
        StepVerifier.create(namesFlux)
                .expectNext("D","E","F","A","U","L","T")
                .verifyComplete();
    }

    @Test
    void exploreConcat(){
        var exploreConcat=fluxAndMOnoGeneratorService.exploreConcat();
        StepVerifier.create(exploreConcat)
                .expectNext("A","B","C","D","E","F")
                .verifyComplete();
    }

    @Test
    void exploreConcatWithMono(){
        var exploreConcatWithMono=fluxAndMOnoGeneratorService.exploreConcatWithMono();
        StepVerifier.create(exploreConcatWithMono)
                .expectNext("A","D")
                .verifyComplete();
    }
    @Test
    void exploreMerge(){
        var exploreMerge=fluxAndMOnoGeneratorService.exploreMerge();
        StepVerifier.create(exploreMerge)
                .expectNext("A","D","B","E","C","F")
                .verifyComplete();
    }

    @Test
    void exploreMergeWithMono(){
        var exploreMergeWithMono=fluxAndMOnoGeneratorService.exploreMergeWithMono();
        StepVerifier.create(exploreMergeWithMono)
                .expectNext("A","D")
                .verifyComplete();
    }

    @Test
    void exploreMergeSequential(){
        var exploreMergeSequential=fluxAndMOnoGeneratorService.exploreMergeSequential();
        StepVerifier.create(exploreMergeSequential)
                .expectNext("A","B","C","D","E","F")
                .verifyComplete();
    }

    @Test
    void exploreZip(){
        var exploreZip=fluxAndMOnoGeneratorService.exploreZip();
        StepVerifier.create(exploreZip)
                .expectNext("AD","BE","CF")
                .verifyComplete();
    }

    @Test
    void exploreZip1(){
        var exploreZip1=fluxAndMOnoGeneratorService.exploreZip1();
        StepVerifier.create(exploreZip1)
                .expectNext("AD14","BE25","CF36")
                .verifyComplete();
    }

    @Test
    void exploreZipWith(){
        var exploreZip=fluxAndMOnoGeneratorService.exploreZipWith();
        StepVerifier.create(exploreZip)
                .expectNext("AD","BE","CF")
                .verifyComplete();
    }
}

